﻿// ----- Text Filtering Example
//       by Tim Patrick
//       for Visual Studio Magazine

using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Linq;
using System.Runtime.CompilerServices;

namespace FilterList
{
    public class CityData : INotifyPropertyChanged
    {
        private List<CityEntry> AllCities;

        private ObservableCollection<CityEntry> MatchingCitiesInternal;
        public ObservableCollection<CityEntry> MatchingCities
        {
            get { return this.MatchingCitiesInternal; }
            set
            {
                this.MatchingCitiesInternal = value;
                NotifyPropertyChanged();
            }
        }

        private string LastFilter = "";
        public string Filter
        {
            // ----- String pattern that filters the cities.
            get
            {
                return this.LastFilter;
            }
            set
            {
                if (value != this.LastFilter)
                {
                    // ----- Update the filtered list.
                    this.LastFilter = value ?? "";
                    this.MatchingCities.Clear();
                    if (this.LastFilter.Trim().Length == 0)
                    {
                        // ----- No filters. Add all cities.
                        foreach (CityEntry oneCity in this.AllCities)
                            this.MatchingCities.Add(oneCity);
                    }
                    else
                    {
                        // ----- Limit cities by filter.
                        var filteredCities =
                            from oneCity in this.AllCities
                            where oneCity.IsMatch(this.LastFilter)
                            select oneCity;
                        foreach (CityEntry oneCity in filteredCities)
                            this.MatchingCities.Add(oneCity);
                    }
                    NotifyPropertyChanged();
                }
            }
        }

        public CityData()
        {
            this.AllCities = new List<CityEntry>()
            {
                // ----- Initialize with interesting data. Source: localistica.com
                new CityEntry("Albuquerque, NM", "87121"),
                new CityEntry("Anaheim, CA", "92804"),
                new CityEntry("Antioch, TN", "37013"),
                new CityEntry("Atlanta, GA", "30349"),
                new CityEntry("Bakersfield, CA", "93307"),
                new CityEntry("Baldwin Park, CA", "91706"),
                new CityEntry("Bell Gardens, CA", "90201"),
                new CityEntry("Bellflower, CA", "90706"),
                new CityEntry("Bronx, NY", "10452", "10453", "10456", "10458", "10462", "10467", "10468"),
                new CityEntry("Brooklyn, NY", "11206", "11207", "11212", "11214", "11219", "11226", "11230", "11234", "11235"),
                new CityEntry("Brownsville, TX", "78521"),
                new CityEntry("Charlotte, NC", "28269"),
                new CityEntry("Chicago, IL", "60617", "60618", "60625", "60629", "60647"),
                new CityEntry("Chino Hills, CA", "91709"),
                new CityEntry("Chino, CA", "91710"),
                new CityEntry("Chula Vista, CA", "91910", "91911"),
                new CityEntry("Corona, NY", "11368"),
                new CityEntry("Cypress, TX", "77429"),
                new CityEntry("Dallas, TX", "75211", "75217"),
                new CityEntry("El Paso, TX", "79912", "79936"),
                new CityEntry("Elmhurst, NY", "11373"),
                new CityEntry("Flushing, NY", "11355"),
                new CityEntry("Fontana, CA", "92335", "92336"),
                new CityEntry("Fresno, CA", "93722"),
                new CityEntry("Grand Prairie, TX", "75052"),
                new CityEntry("Hawthorne, CA", "90250"),
                new CityEntry("Hayward, CA", "94544"),
                new CityEntry("Hesperia, CA", "92345"),
                new CityEntry("Houston, TX", "77036", "77083", "77084"),
                new CityEntry("Huntington Park, CA", "90255"),
                new CityEntry("Katy, TX", "77449", "77494"),
                new CityEntry("La Puente, CA", "91744"),
                new CityEntry("Lakewood, NJ", "08701"),
                new CityEntry("Lancaster, CA", "93535"),
                new CityEntry("Las Vegas, NV", "89110"),
                new CityEntry("Lawrenceville, GA", "30043", "30044"),
                new CityEntry("League City, TX", "77573"),
                new CityEntry("Long Beach, CA", "90805"),
                new CityEntry("Los Angeles, CA", "90011", "90044"),
                new CityEntry("Mckinney, TX", "75070"),
                new CityEntry("Mission, TX", "78572"),
                new CityEntry("Moreno Valley, CA", "92553"),
                new CityEntry("Nashville, TN", "37211"),
                new CityEntry("New York, NY", "10002", "10025", "10029"),
                new CityEntry("Norwalk, CA", "90650"),
                new CityEntry("Olathe, KS", "66062"),
                new CityEntry("Oxnard, CA", "93033"),
                new CityEntry("Pacoima, CA", "91331"),
                new CityEntry("Palmdale, CA", "93550"),
                new CityEntry("Pearland, TX", "77584"),
                new CityEntry("Pharr, TX", "78577"),
                new CityEntry("Pittsburg, CA", "94565"),
                new CityEntry("Porterville, CA", "93257"),
                new CityEntry("Reseda, CA", "91335"),
                new CityEntry("Rialto, CA", "92376"),
                new CityEntry("Riverside, CA", "92503", "92509"),
                new CityEntry("Sacramento, CA", "95823"),
                new CityEntry("San Diego, CA", "92154"),
                new CityEntry("San Juan, PR", "00926"),
                new CityEntry("Santa Ana, CA", "92704"),
                new CityEntry("Simi Valley, CA", "93065"),
                new CityEntry("South Gate, CA", "90280"),
                new CityEntry("Sugar Land, TX", "77479"),
                new CityEntry("Sylmar, CA", "91342"),
                new CityEntry("Virginia Beach, VA", "23464"),
                new CityEntry("Watsonville, CA", "95076"),
                new CityEntry("Westminster, CA", "92683"),
                new CityEntry("Woodbridge, VA", "22193"),
                new CityEntry("Woodside, NY", "11377"),
                new CityEntry("Yuma, AZ", "85364")
            };

            // ----- All cities match by default.
            this.MatchingCities = new ObservableCollection<CityEntry>();
            foreach (CityEntry oneCity in this.AllCities)
                this.MatchingCities.Add(oneCity);
        }

        // ----- Boilerplate for notifications.
        public event PropertyChangedEventHandler PropertyChanged;
        private void NotifyPropertyChanged([CallerMemberName] string propertyName = "")
        {
            PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(propertyName));
        }
    }
}
